#include <vector>
#include <iostream>


struct __attribute__ ((__packed__))  LeCroy_time           // Time Stamp structure for trigger time from LeCroy scope
{       double          seconds;                         // seconds and fractional seconds
        char            minutes;                         // (0-59)
        char            hours;                                   // (0-23)
        char            days;                                    // (0-31)
        char            months;                                  // (1-12)
        unsigned short int           year;                                    // (0-16000)
        unsigned short int            unused;                                  //
};



struct __attribute__ ((__packed__)) Header
{
        char            DEF9header[11];          // 'DESC,#9000000346'  (Not part of template)
        char            DESCRIPTOR_NAME[16];     // 'WAVEDESC        '
        char            TEMPLATE_NAME[16];       // 'LECROY_2_3      ' Name of the template
        unsigned short int           COMM_TYPE;                               // 0=byte, 1=word
        unsigned short int           COMM_ORDER;                      // 0=hifirst, 1=lofirst
        unsigned int           WAVE_DESCRIPTOR;         // length of block WAVEDESC
        unsigned int           USER_TEXT;                               // length of block USERTEXT
        unsigned int           RES_DESC1;                              // unknown
        unsigned int           TRIGTIME_ARRAY;          // length of TRIGTIME array
        unsigned int           RIS_TIME_ARRAY;          // length of RIS_TIME array
        unsigned int           RES_ARRAY1;                      //    (expansion entry)
        unsigned int           WAVE_ARRAY_1;                    // lengh of first data array
        unsigned int           WAVE_ARRAY_2;                    // length of second data array
        unsigned int           RES_ARRAY2;                      //    (expansion entry)
        unsigned int           RES_ARRAY3;                      //    (expansion entry)
        char            INSTRUMENT_NAME[16];     // string name of instrument
        unsigned int           INSTRUMENT_NUMBER;       // serial number
        char            TRACE_LABEL[16];         // identifies the waveform
        unsigned short int           RESERVED1;                               //    (expansion entry)
        unsigned short int           RESERVED2;                               //    (expansion entry)
        unsigned int           WAVE_ARRAY_COUNT;                // number of data points in data array
        unsigned int           PNTS_PER_SCREEN;         // nominal number of points on screen
        unsigned int           FIRST_VALID_PNT;         // number points to skip before first valid
        unsigned int           LAST_VALID_PNT;          // index of last good point
        unsigned int           FIRST_POINT;                     // offset from beginning of trace buffer
        unsigned int           SPARSING_FACTOR;         // sparsing into the data block
        unsigned int           SEGMENT_INDEX;           // index of the transmitted segment
        unsigned int           SUBARRAY_COUNT;          // acquired segment count
        unsigned int           SWEEPS_PER_ACQ;          // for average or extrama
        unsigned short int           POINTS_PER_PAIR;         // for peak detect
        unsigned short int           PAIR_OFFSET;                     // for peak detect, offset array2 to array1
        float           VERTICAL_GAIN;           // to get floating values from raw data:
        float           VERTICAL_OFFSET;         //     value = VERTICAL_GAIN * data - VERTICAL_OFFSET
        float           MAX_VALUE;                               // maximum allowed value, upper edge of the grid.
        float           MIN_VALUE;                               // minimum allowed value, lower edge of the grid.
        unsigned short int           NOMINAL_BITS;                    // intrinsic precision: ADC data is 8 bit, averate 10-12
        unsigned short int           NOM_SUBARRAY_COUNT;      // for Sequence, nominal segment count, else 1
        float           HORIZ_INTERVAL;          // sampling interval for time domain waveforms
        double          HORIZ_OFFSET;                    // seconds between trigger and the first data point
        double          PIXEL_OFFSET;                    // "needed to know how to display the waveform"
        char            VERTUNIT[48];                    // units of the vertical axis
        char            HORUNIT[48];                     // units of the horizontal axis
        float           HORIZ_UNCERTAINTY;       // uncertainty of the horizontal offset in seconds
        LeCroy_time     TRIGGER_TIME;    // time of the trigger
        float           ACQ_DURATION;                    // duration of the acquisition (in sec)
        unsigned short int           RECORD_TYPE;                     // 0=single_sweep, etc
        unsigned short int           PROCESSING_DONE;         // 0=no_processing, etc
        unsigned short int           RESERVED5;                               // (expansion entry)
        unsigned short int           RIS_SWEEPS;                      // for RIS, number of sweeps, else 1
        unsigned short int           TIMEBASE;                                // 0=1_ps/div, 1=2_ps/div, etc
        unsigned short int           VERT_COUPLING;           // 0=DC_50_Ohms, 1=ground, 2=DC_1MOhm, 3=ground, 4=AC_1MOhm
        float           PROBE_ATT;                               // probe attenuation
        unsigned short int           FIXED_VERT_GAIN;         // 0=1_uV/div, 1=2_uv/div, etc
        unsigned short int           BANDWIDTH_LIMIT;         // 0=off, 1=on
        float           VERTICAL_VERNIER;                // vertical gain fine adjust
        float           ACQ_VERT_OFFSET;         // unknown
        unsigned short int           WAVE_SOURCE;                     // 0=CHANNEL_1, 1=CHANNEL_2, 2=CHANNEL_3, 3=CHANNEL_4, 9=UNKNOWN
} ;


short int data2;
char data1;



//void gengrapf(TString filename, Header header, TGraph * gr) 
void gengraph(TString filename, Header header, TGraph *&gr) 
{

std::vector<Double_t> index;
std::vector<Double_t> amplitude;

index.clear();
amplitude.clear();

Int_t nC = 0;
ifstream C;

C.open(filename,ios::in | ios::binary);
C.read((char*)&header, sizeof(Header)); 
                    
 std::cout<<header.HORIZ_INTERVAL<<std::endl;

 
 while (1) {
    if(header.COMM_TYPE)
      { C.read((char*)&data2, sizeof(data2));}
    else
      { C.read((char*)&data1, sizeof(data1));}

   if (!C.good()) break;
       nC++;
      index.push_back(nC);
    if(header.COMM_TYPE)
      {amplitude.push_back((data2));}
    else
      {amplitude.push_back((data1));}
   }//while

C.close();

gr = new TGraph(index.size(), &(index[0]) ,&(amplitude[0]));


};


void setgraph(TGraph *&gr, Color_t  lcolor=1, Width_t lwidth=1,Style_t lstyle=1,Color_t mcolor = 1,Size_t msize = 1, Style_t mstyle = 1, TString title="",
TString xaxis="x-axis", TString yaxis="y-axis")
{
gr->SetLineColor(lcolor);
gr->SetLineWidth(lwidth);
gr->SetLineStyle(lstyle);
gr->SetMarkerColor(mcolor);
gr->SetMarkerSize(msize);
gr->SetMarkerStyle(mstyle);
gr->SetTitle(title);
gr->GetXaxis()->SetTitle(xaxis);
gr->GetYaxis()->SetTitle(yaxis);
}



void dekoder(TString filename1 = "Trace00000")
{
  std::cout<<"Procesing WaveForm Tracing files"<<std::endl;
  TString     filenameC1="C1"+filename1+".trc";
  TString     filenameC2="C2"+filename1+".trc";
  std::cout<<filenameC1<<std::endl;
  std::cout<<filenameC2<<std::endl;

  // TString     filenameC3="C3"+filename1+".trc";
  // TString     filenameC4="C4"+filename1+".trc";
 

  Header headerC11,headerC21,headerC31,headerC41;
  TGraph *grC11,*grC21,*grC31,*grC41;

  gengraph(filenameC1,headerC11,grC11);
  gengraph(filenameC2,headerC21,grC21);
  // gengraph(filenameC3,headerC31,grC31);
  // gengraph(filenameC4,headerC41,grC41);

  setgraph(grC11,kBlack,2,1,kBlack,.6,kFullCircle,"C11","Point","C1");
  setgraph(grC21,kRed,2,1,kRed,.6,kFullCircle,"C21","Point","C2");
  // setgraph(grC31,kBlue,2,1,kBlue,.6,kFullCircle,"C31","Point","C3");
  // setgraph(grC41,kGreen,2,1,kGreen,.6,kFullCircle,"C41","Point","C4");

  auto *hC11 = (TH1F*) grC11->GetHistogram();
  for (int i=0; i<grC11->GetN(); i++)
    hC11->SetBinContent(i+1, grC11->GetPointY(i));
  
  TMultiGraph *mg1 = new TMultiGraph();

  mg1->Add(grC11);
  mg1->Add(grC21);
  // mg1->Add(grC31);
  // mg1->Add(grC41);

  
  TCanvas *c1 = new TCanvas("c1",filename1);

   c1->Divide(1,2);
    TVirtualPad *c1_1 = c1->cd(1);
    c1_1->SetGrid();
    TVirtualPad *c1_2 = c1->cd(2);
    c1_2->SetGrid();
    c1->cd(1);
    grC11->Draw("APL");
    c1->cd(2);
    grC21->Draw("APL");
    c1->Update();
    c1->Print(filename1+".png"); 


 
}
